@@ -11,8 +11,8 @@
 ################################################################################
 
 INSTALLER_NAME="Rootkit Hunter installer"
-INSTALLER_VERSION="1.2.13"
-INSTALLER_COPYRIGHT="Copyright 2003-2009, Michael Boelen"
+INSTALLER_VERSION="1.2.15"
+INSTALLER_COPYRIGHT="Copyright 2003-2010, Michael Boelen"
 INSTALLER_LICENSE="
 
 Under active development by the Rootkit Hunter project team. For reporting
@@ -24,7 +24,7 @@ of the GNU General Public License. See LICENSE for details.
 "
 
 APPNAME="rkhunter"
-APPVERSION="1.3.6"
+APPVERSION="1.3.8"
 RKHINST_OWNER="0:0"
 RKHINST_MODE_EX="0750"
 RKHINST_MODE_RW="0640"
@@ -37,6 +37,10 @@ ERRCODE=0
 OVERWRITE=0
 STRIPROOT=""
 
+RPM_USING_ROOT=0
+TGZ_USING_ROOT=0
+DEB_USING_ROOT=0
+
 umask 027
 
 OPERATING_SYSTEM=`uname`
@@ -120,7 +124,7 @@ showExamples() { # Show examples
 
 showVersion() { echo "${INSTALLER_NAME} ${INSTALLER_VERSION} ${INSTALLER_LICENSE}"; return; }
 
-selectTemplate() { # Take input from the "--installdir parameter"
+selectTemplate() { # Take input from the "--install parameter"
 	case "$1" in
 	/usr|/usr/local|default|custom_*|RPM|DEB|TGZ)
 		case "$1" in
@@ -128,7 +132,7 @@ selectTemplate() { # Take input from the "--installdir parameter"
 			PREFIX="/usr/local"
 			;;
 		custom_*)
-			PREFIX=`echo "${RKHINST_LAYOUT}" | sed 's/custom_//g'`
+			PREFIX=`echo "${RKHINST_LAYOUT}" | sed 's|custom_||g'`
 
 			case "${PREFIX}" in
 			.)
@@ -136,11 +140,13 @@ selectTemplate() { # Take input from the "--installdir parameter"
 					echo "Standalone installation into ${PWD}/files"
 				fi
 				;;
-			.*|/.*)
-				echo "Bad installation directory chosen. Exiting."
+			.*|/.*|*//*)
+				echo "Invalid layout directory chosen: ${PREFIX}"
 				exit 1
 				;;
 			*)
+				test "${PREFIX}" = "/" && PREFIX=""
+
 				if [ "${RKHINST_ACTION}" = "install" ]; then
 					RKHTMPVAR=`echo "${PATH}" | grep "${PREFIX}/bin"`
 
@@ -155,25 +161,40 @@ selectTemplate() { # Take input from the "--installdir parameter"
 			;;
 		RPM)
 			if [ -n "${RPM_BUILD_ROOT}" ]; then
-				PREFIX="${RPM_BUILD_ROOT}/usr/local"
+				if [ "${RPM_BUILD_ROOT}" = "/" ]; then
+					RPM_USING_ROOT=1
+					PREFIX="/usr/local"
+				else
+					PREFIX="${RPM_BUILD_ROOT}/usr/local"
+				fi
 			else
-				echo "RPM installation chosen but \$RPM_BUILD_ROOT variable not found: exiting."
+				echo "RPM installation chosen but \$RPM_BUILD_ROOT variable not found. Exiting."
 				exit 1
 			fi
 			;;
 		DEB)
 			if [ -n "${DEB_BUILD_ROOT}" ]; then
-				PREFIX="${DEB_BUILD_ROOT}/usr"
+				if [ "${DEB_BUILD_ROOT}" = "/" ]; then
+					DEB_USING_ROOT=1
+					PREFIX="/usr"
+				else
+					PREFIX="${DEB_BUILD_ROOT}/usr"
+				fi
 			else
-				echo "DEB installation chosen but \$DEB_BUILD_ROOT variable not found: exiting."
+				echo "DEB installation chosen but \$DEB_BUILD_ROOT variable not found. Exiting."
 				exit 1
 			fi
 			;;
 		TGZ)
 			if [ -n "${TGZ_BUILD_ROOT}" ]; then
-				PREFIX="${TGZ_BUILD_ROOT}/usr"
+				if [ "${TGZ_BUILD_ROOT}" = "/" ]; then
+					TGZ_USING_ROOT=1
+					PREFIX="/usr"
+				else
+					PREFIX="${TGZ_BUILD_ROOT}/usr"
+				fi
 			else
-				echo "TGZ installation chosen but \$TGZ_BUILD_ROOT variable not found: exiting."
+				echo "TGZ installation chosen but \$TGZ_BUILD_ROOT variable not found. Exiting."
 				exit 1
 			fi
 			;;
@@ -187,8 +208,8 @@ selectTemplate() { # Take input from the "--installdir parameter"
 			;;
 		*)
 			if [ "${RKHINST_ACTION}" = "install" ]; then
-				if [ ! -d "${PREFIX}" ]; then
-					echo "Bad installation directory chosen: non-existent directory. Exiting."
+				if [ -n "${PREFIX}" -a ! -d "${PREFIX}" ]; then
+					echo "Non-existent installation directory chosen: ${PREFIX}"
 					echo "Perhaps run \"mkdir -p ${PREFIX}\" first?"
 					exit 1
 				fi
@@ -201,13 +222,25 @@ selectTemplate() { # Take input from the "--installdir parameter"
 			SYSCONFIGDIR="${PREFIX}/etc"
 			;;
 		RPM)
-			SYSCONFIGDIR="${RPM_BUILD_ROOT}/etc"
+			if [ $RPM_USING_ROOT -eq 1 ]; then
+				SYSCONFIGDIR="/etc"
+			else
+				SYSCONFIGDIR="${RPM_BUILD_ROOT}/etc"
+			fi
 			;;
 		DEB)
-			SYSCONFIGDIR="${DEB_BUILD_ROOT}/etc"
+			if [ $DEB_USING_ROOT -eq 1 ]; then
+				SYSCONFIGDIR="/etc"
+			else
+				SYSCONFIGDIR="${DEB_BUILD_ROOT}/etc"
+			fi
 			;;
 		TGZ)
-			SYSCONFIGDIR="${TGZ_BUILD_ROOT}/etc"
+			if [ $TGZ_USING_ROOT -eq 1 ]; then
+				SYSCONFIGDIR="/etc"
+			else
+				SYSCONFIGDIR="${TGZ_BUILD_ROOT}/etc"
+			fi
 			;;
 		*)
 			SYSCONFIGDIR="/etc"
@@ -216,42 +249,77 @@ selectTemplate() { # Take input from the "--installdir parameter"
 
 		case "$1" in
 		custom_*)
-			if [ "${UNAMEM}" = "x86_64" ]; then
-				LIBDIR="${PREFIX}/lib64"
-			else
-				LIBDIR="${PREFIX}/lib"
+			LIBDIR=""
+
+			if [ -z "${PREFIX}" -a "${OPERATING_SYSTEM}" = "Darwin" ]; then
+				test -d "/Library" && LIBDIR="/Library"
+			fi
+
+			if [ -z "${LIBDIR}" ]; then
+				if [ "${UNAMEM}" = "x86_64" -o "${UNAMEM}" = "ppc64" ]; then
+					LIBDIR="${PREFIX}/lib64"
+				else
+					LIBDIR="${PREFIX}/lib"
+				fi
 			fi
 
 			BINDIR="${PREFIX}/bin"
 			VARDIR="${PREFIX}/var"
-			SHAREDIR="${PREFIX}/share"
+
+			if [ -z "${PREFIX}" ]; then
+				if [ ! -d "/share" -a -d "/usr/share" ]; then
+					SHAREDIR="/usr/share"
+				else
+					SHAREDIR="/share"
+				fi
+			else
+				SHAREDIR="${PREFIX}/share"
+			fi
 			;;
 		RPM)
-			if [ "${UNAMEM}" = "x86_64" ]; then
+			if [ "${UNAMEM}" = "x86_64" -o "${UNAMEM}" = "ppc64" ]; then
 				LIBDIR="${PREFIX}/lib64"
 			else
 				LIBDIR="${PREFIX}/lib"
 			fi
 
 			BINDIR="${PREFIX}/bin"
-			VARDIR="${RPM_BUILD_ROOT}/var"
+
+			if [ $RPM_USING_ROOT -eq 1 ]; then
+				VARDIR="/var"
+			else
+				VARDIR="${RPM_BUILD_ROOT}/var"
+			fi
+
 			SHAREDIR="${PREFIX}/share"
 			;;
 		DEB)
 			BINDIR="${PREFIX}/bin"
 			LIBDIR="${PREFIX}/lib"
-			VARDIR="${DEB_BUILD_ROOT}/var"
+
+			if [ $DEB_USING_ROOT -eq 1 ]; then
+				VARDIR="/var"
+			else
+				VARDIR="${DEB_BUILD_ROOT}/var"
+			fi
+
 			SHAREDIR="${PREFIX}/share"
 			;;
 		TGZ)
-			if [ "${UNAMEM}" = "x86_64" ]; then
+			if [ "${UNAMEM}" = "x86_64" -o "${UNAMEM}" = "ppc64" ]; then
 				LIBDIR="${PREFIX}/lib64"
 			else
 				LIBDIR="${PREFIX}/lib"
 			fi
 
 			BINDIR="${PREFIX}/bin"
-			VARDIR="${TGZ_BUILD_ROOT}/var"
+
+			if [ $TGZ_USING_ROOT -eq 1 ]; then
+				VARDIR="/var"
+			else
+				VARDIR="${TGZ_BUILD_ROOT}/var"
+			fi
+
 			SHAREDIR="${PREFIX}/share"
 			;;
 		*)
@@ -277,7 +345,7 @@ selectTemplate() { # Take input from the "--installdir parameter"
 		BINDIR="${PREFIX}/bin"
 		;;
 	*)	# None chosen.
-		echo "No template chosen, exiting."
+		echo "No template chosen. Exiting."
 		exit 1
 		;;
 	esac
@@ -307,7 +375,7 @@ selectTemplate() { # Take input from the "--installdir parameter"
 	RKHINST_ETC_FILE="${APPNAME}.conf"
 	RKHINST_BIN_FILES="${APPNAME}"
 
-	RKHINST_SCRIPT_FILES="check_modules.pl filehashmd5.pl filehashsha1.pl filehashsha.pl stat.pl readlink.sh"
+	RKHINST_SCRIPT_FILES="check_modules.pl filehashsha.pl stat.pl readlink.sh"
 	RKHINST_DB_FILES="backdoorports.dat mirrors.dat programs_bad.dat suspscan.dat"
 
 	if [ "${RKHINST_LAYOUT}" = "DEB" ]; then
@@ -328,29 +396,67 @@ selectTemplate() { # Take input from the "--installdir parameter"
 # /etc/sysconfig/rkhunter (config for cronjob)
 # /etc/logrotate.d/rkhunter
 
-showTemplate() { # Take input from the "--installdir parameter"
+showTemplate() { # Take input from the "--install parameter"
 	case "$1" in
 	custom_.)
 		# Dump *everything* in the current dir.
 		echo "Standalone installation into ${PWD}/files"
 		;;
 	*)
+		NOTPRESENT="   (Directory will be created)"
+
 		selectTemplate "$1"
 
-		echo "Install into:       ${PREFIX}"
-		echo "Application:        ${RKHINST_BIN_DIR}"
+		test -z "${PREFIX}" && RKHTMPVAR="/" || RKHTMPVAR="${PREFIX}"
 
+		test ! -d "${RKHTMPVAR}" && RKHTMPVAR="${RKHTMPVAR}                            (Directory does not exist)"
+		echo "Install into:       ${RKHTMPVAR}"
+
+		RKHTMPVAR="${RKHINST_BIN_DIR}"
+		test ! -d "${RKHTMPVAR}" && RKHTMPVAR="${RKHTMPVAR}                     ${NOTPRESENT}"
+		echo "Application:        ${RKHTMPVAR}"
+
+		RKHTMPVAR="${RKHINST_ETC_DIR}"
+		test ! -d "${RKHTMPVAR}" && RKHTMPVAR="${RKHTMPVAR}                     ${NOTPRESENT}"
 		if [ $OVERWRITE -eq 0 ]; then
-			echo "Configuration file: ${RKHINST_ETC_DIR}"
+			echo "Configuration file: ${RKHTMPVAR}"
 		else
-			echo "Configuration file: ${RKHINST_ETC_DIR}   (Configuration file will be overwritten)"
+			if [ ! -d "${RKHTMPVAR}" ]; then
+				echo "Configuration file: ${RKHTMPVAR}"
+			else
+				if [ -z "${PREFIX}" ]; then
+					echo "Configuration file: ${RKHINST_ETC_DIR}               (Configuration file will be overwritten)"
+				else
+					echo "Configuration file: ${RKHINST_ETC_DIR}           (Configuration file will be overwritten)"
+				fi
+			fi
 		fi
 
-		echo "Documents:          ${RKHINST_DOC_DIR}"
-		echo "Man page:           ${RKHINST_MAN_DIR}"
-		echo "Scripts:            ${RKHINST_SCRIPT_DIR}"
-		echo "Databases:          ${RKHINST_DB_DIR}"
-		echo "Temporary files:    ${RKHINST_TMP_DIR}"
+		RKHTMPVAR="${RKHINST_DOC_DIR}"
+		test ! -d "${RKHTMPVAR}" && RKHTMPVAR="${RKHTMPVAR}${NOTPRESENT}"
+		echo "Documents:          ${RKHTMPVAR}"
+
+		RKHTMPVAR="${RKHINST_MAN_DIR}"
+		test ! -d "${RKHTMPVAR}" && RKHTMPVAR="${RKHTMPVAR}          ${NOTPRESENT}"
+		echo "Man page:           ${RKHTMPVAR}"
+
+		RKHTMPVAR="${RKHINST_SCRIPT_DIR}"
+		if [ ! -d "${RKHTMPVAR}" ]; then
+			if [ -z "${PREFIX}" ]; then
+				RKHTMPVAR="${RKHTMPVAR}      ${NOTPRESENT}"
+			else
+				RKHTMPVAR="${RKHTMPVAR}  ${NOTPRESENT}"
+			fi
+		fi
+		echo "Scripts:            ${RKHTMPVAR}"
+
+		RKHTMPVAR="${RKHINST_DB_DIR}"
+		test ! -d "${RKHTMPVAR}" && RKHTMPVAR="${RKHTMPVAR}     ${NOTPRESENT}"
+		echo "Databases:          ${RKHTMPVAR}"
+
+		RKHTMPVAR="${RKHINST_TMP_DIR}"
+		test ! -d "${RKHTMPVAR}" && RKHTMPVAR="${RKHTMPVAR}    ${NOTPRESENT}"
+		echo "Temporary files:    ${RKHTMPVAR}"
 
 		if [ -n "${STRIPROOT}" ]; then
 			echo ""; echo "Got STRIPROOT=\"${STRIPROOT}\""
@@ -370,7 +476,7 @@ useCVS() {
 		echo "Unable to find the 'cvs' command."
 		exit 1
 	else
-		cvs -z3 -d:pserver:anonymous@rkhunter.cvs.sourceforge.net:/cvsroot/rkhunter co rkhunter
+		cvs -z3 -d:pserver:anonymous@rkhunter.cvs.sourceforge.net:/cvsroot/rkhunter co rkhunter >/dev/null 2>&1
 		ERRCODE=$?
 
 		if [ $ERRCODE -eq 0 ]; then
@@ -396,7 +502,7 @@ useCVS() {
 				;;
 			esac
 		else
-			echo "Failed to get Rootkit Hunter from CVS: code $ERRCODE: exiting."
+			echo "Failed to get Rootkit Hunter from CVS: code $ERRCODE"
 			exit 1
 		fi
 	fi
@@ -479,7 +585,9 @@ doInstall()  {
 		;;
 	*) 
 		# Check PREFIX
-		if [ -d "${PREFIX}" ]; then
+		if [ -z "${PREFIX}" ]; then
+			:
+		elif [ -d "${PREFIX}" ]; then
 			if [ -w "${PREFIX}" ]; then
 				echo " Checking installation directory \"$PREFIX\": it exists and is writable."
 
@@ -554,20 +662,20 @@ doInstall()  {
 	umask 022
 	for DIR in ${RKHDIR_LIST}; do
 		if [ -d "${DIR}" ]; then
-			if [ -w "${PREFIX}" ]; then
+			if [ -w "${DIR}" ]; then
 				echo "  Directory ${DIR}: exists and is writable."
 			else
 				echo "  Directory ${DIR}: exists, but it is not writable. Exiting."
 				exit 1
 			fi
 		else
-			mkdir -p ${DIR}
+			mkdir -p ${DIR} >/dev/null 2>&1
 			ERRCODE=$?
 
 			if [ $ERRCODE -eq 0 ]; then
 				echo "  Directory ${DIR}: creating: OK"
 			else
-				echo "  Directory ${DIR}: creating: FAILED: Code $ERRCODE: exiting."
+				echo "  Directory ${DIR}: creating: FAILED: Code $ERRCODE"
 				exit 1
 			fi
 		fi
@@ -576,20 +684,20 @@ doInstall()  {
 
 	for DIR in ${RKHINST_DIRS_EXCEP}; do
 		if [ -d "${DIR}" ]; then
-			if [ -w "${PREFIX}" ]; then
+			if [ -w "${DIR}" ]; then
 				echo "  Directory ${DIR}: exists and is writable."
 			else
 				echo "  Directory ${DIR}: exists, but it is not writable. Exiting."
 				exit 1
 			fi
 		else
-			mkdir -p "${DIR}"
+			mkdir -p ${DIR} >/dev/null 2>&1
 			ERRCODE=$?
 
 			if [ $ERRCODE -eq 0 ]; then
 				echo "  Directory ${DIR}: creating: OK"
 			else
-				echo "  Directory ${DIR}: creating: FAILED: Code $ERRCODE: exiting."
+				echo "  Directory ${DIR}: creating: FAILED: Code $ERRCODE"
 				exit 1
 			fi
 		fi
@@ -610,14 +718,14 @@ doInstall()  {
 	for FILE in ${RKHINST_SCRIPT_FILES} ${RKHINST_DB_FILES} ${RKHINST_MAN_FILES}; do
 		case "${FILE}" in
 		*.pl|*.sh)
-			cp -f ./files/"${FILE}" "${RKHINST_SCRIPT_DIR}"
+			cp -f ./files/"${FILE}" "${RKHINST_SCRIPT_DIR}" >/dev/null 2>&1
 			ERRCODE=$?
 
 			if [ $ERRCODE -eq 0 ]; then
 				echo " Installing ${FILE}: OK"
 				chmod "${RKHINST_MODE_EX}" "${RKHINST_SCRIPT_DIR}/${FILE}"
 			else
-				echo " Installing ${FILE}: FAILED: Code $ERRCODE: exiting."
+				echo " Installing ${FILE}: FAILED: Code $ERRCODE"
 				exit 1
 			fi
 			;;
@@ -631,26 +739,26 @@ doInstall()  {
 				fi
 			fi
 
-			cp -f ./files/"${FILE}" "${RKHINST_DB_DIR}"
+			cp -f ./files/"${FILE}" "${RKHINST_DB_DIR}" >/dev/null 2>&1
 			ERRCODE=$?
 
 			if [ $ERRCODE -eq 0 ]; then
 				echo " Installing ${FILE}: OK"
 				chmod "${RKHINST_MODE_RW}" "${RKHINST_DB_DIR}/${FILE}"
 			else
-				echo " Installing ${FILE}: FAILED: Code $ERRCODE: exiting."
+				echo " Installing ${FILE}: FAILED: Code $ERRCODE"
 				exit 1
 			fi
 			;;
 		*.8)
-			cp -f ./files/"${FILE}" "${RKHINST_MAN_DIR}"
+			cp -f ./files/"${FILE}" "${RKHINST_MAN_DIR}" >/dev/null 2>&1
 			ERRCODE=$?
 
 			if [ $ERRCODE -eq 0 ]; then
 				echo " Installing ${FILE}: OK"
 				chmod "${RKHINST_MODE_RWR}" "${RKHINST_MAN_DIR}/${FILE}"
 			else
-				echo " Installing ${FILE}: FAILED: Code $ERRCODE: exiting."
+				echo " Installing ${FILE}: FAILED: Code $ERRCODE"
 				exit 1
 			fi
 			;;
@@ -660,14 +768,14 @@ doInstall()  {
 
 	# Application documents
 	for FILE in ${RKHINST_DOC_FILES}; do
-		cp -f ./files/"${FILE}" "${RKHINST_DOC_DIR}"
+		cp -f ./files/"${FILE}" "${RKHINST_DOC_DIR}" >/dev/null 2>&1
 		ERRCODE=$?
 
 		if [ $ERRCODE -eq 0 ]; then
 			echo " Installing ${FILE}: OK"
 			chmod "${RKHINST_MODE_RWR}" "${RKHINST_DOC_DIR}/${FILE}"
 		else
-			echo " Installing ${FILE}: FAILED: Code $ERRCODE: exiting."
+			echo " Installing ${FILE}: FAILED: Code $ERRCODE"
 			exit 1
 		fi
 	done
@@ -677,7 +785,7 @@ doInstall()  {
 	ERRCODE=0
 
 	for FILE in `find ./files/i18n -type f`; do
-		cp "${FILE}" "${RKHINST_LANG_DIR}"
+		cp "${FILE}" "${RKHINST_LANG_DIR}" >/dev/null 2>&1
 		ERRCODE=$?
 
 		test $ERRCODE -ne 0 && break
@@ -686,7 +794,7 @@ doInstall()  {
 	if [ $ERRCODE -eq 0 ];then
 		echo " Installing language support files: OK"
 	else
-		echo " Installing language support files: FAILED: Code $ERRCODE: exiting."
+		echo " Installing language support files: FAILED: Code $ERRCODE"
 		exit 1
 	fi
 
@@ -695,14 +803,14 @@ doInstall()  {
 	for FILE in ${RKHINST_BIN_FILES}; do
 		case "${RKHINST_LAYOUT}" in
 		RPM|DEB|TGZ)	
-			cp -f ./files/"${FILE}" "${RKHINST_BIN_DIR}/${FILE}"
+			cp -f ./files/"${FILE}" "${RKHINST_BIN_DIR}/${FILE}" >/dev/null 2>&1
 			ERRCODE=$?
 
 			if [ $ERRCODE -eq 0 ]; then
 				echo " Installing ${FILE}: OK"
 				chmod "${RKHINST_MODE_EX}" "${RKHINST_BIN_DIR}/${FILE}"
 			else
-				echo " Installing ${FILE}: FAILED: Code $ERRCODE: exiting."
+				echo " Installing ${FILE}: FAILED: Code $ERRCODE"
 				exit 1
 			fi
 			;;
@@ -714,7 +822,7 @@ doInstall()  {
 				echo " Installing ${FILE}: OK"
 				chmod "${RKHINST_MODE_EX}" "${RKHINST_BIN_DIR}/${FILE}"
 			else
-				echo " Installing ${FILE}: FAILED: Code $ERRCODE: exiting."
+				echo " Installing ${FILE}: FAILED: Code $ERRCODE"
 				exit 1
 			fi
 			;;
@@ -742,19 +850,22 @@ doInstall()  {
 
 			NEWFILE="${FILE}.${RANDVAL}"
 
-			cp -f "./files/${FILE}" "${RKHINST_ETC_DIR}/${NEWFILE}"
+			cp -f "./files/${FILE}" "${RKHINST_ETC_DIR}/${NEWFILE}" >/dev/null 2>&1
 			ERRCODE=$?
 
 			if [ $ERRCODE -eq 0 ]; then
 				echo " Installing ${FILE} in no-clobber mode: OK"
 				chmod "${RKHINST_MODE_RW}" "${RKHINST_ETC_DIR}/${NEWFILE}"
 			else
-				echo " Installing ${FILE} in no-clobber mode: FAILED: Code $ERRCODE: exiting."
+				echo " Installing ${FILE} in no-clobber mode: FAILED: Code $ERRCODE"
 				exit 1
 			fi
 
 			echo "" >>"${RKHINST_ETC_DIR}/${NEWFILE}"
-			echo "INSTALLDIR=${PREFIX}" >>"${RKHINST_ETC_DIR}/${NEWFILE}"
+
+			test -z "${PREFIX}" && RKHTMPVAR="/" || RKHTMPVAR="${PREFIX}"
+
+			echo "INSTALLDIR=${RKHTMPVAR}" >>"${RKHINST_ETC_DIR}/${NEWFILE}"
 			echo "DBDIR=${RKHINST_DB_DIR}" >>"${RKHINST_ETC_DIR}/${NEWFILE}"
 			echo "SCRIPTDIR=${RKHINST_SCRIPT_DIR}" >>"${RKHINST_ETC_DIR}/${NEWFILE}"
 			echo "TMPDIR=${RKHINST_TMP_DIR}" >>"${RKHINST_ETC_DIR}/${NEWFILE}"
@@ -776,34 +887,38 @@ doInstall()  {
 				;;
 			esac
 		else
-			cp -f "./files/${FILE}" "${RKHINST_ETC_DIR}"
+			cp -f "./files/${FILE}" "${RKHINST_ETC_DIR}" >/dev/null 2>&1
 			ERRCODE=$?
 
 			if [ $ERRCODE -eq 0 ]; then
 				echo " Installing ${FILE}: OK"
 				chmod "${RKHINST_MODE_RW}" "${RKHINST_ETC_DIR}/${FILE}"
 			else
-				echo " Installing ${FILE}: FAILED: Code $ERRCODE: exiting."
+				echo " Installing ${FILE}: FAILED: Code $ERRCODE"
 				exit 1
 			fi
 
 			echo "" >>"${RKHINST_ETC_DIR}/${FILE}"
 
-			if [ -n "${RPM_BUILD_ROOT}" ]; then
+			if [ -n "${RPM_BUILD_ROOT}" -a $RPM_USING_ROOT -eq 0 ]; then
 				echo "INSTALLDIR=${PREFIX}" | sed "s|${RPM_BUILD_ROOT}||g" >>"${RKHINST_ETC_DIR}/${FILE}"
 				echo "DBDIR=${RKHINST_DB_DIR}" | sed "s|${RPM_BUILD_ROOT}||g" >>"${RKHINST_ETC_DIR}/${FILE}"
 				echo "SCRIPTDIR=${RKHINST_SCRIPT_DIR}" | sed "s|${RPM_BUILD_ROOT}||g" >>"${RKHINST_ETC_DIR}/${FILE}"
 				echo "TMPDIR=${RKHINST_TMP_DIR}" | sed "s|${RPM_BUILD_ROOT}||g" >>"${RKHINST_ETC_DIR}/${FILE}"
 				echo "USER_FILEPROP_FILES_DIRS=${RKHINST_ETC_DIR}/${FILE}" | sed "s|${RPM_BUILD_ROOT}||g" >>"${RKHINST_ETC_DIR}/${FILE}"
-			elif [ -n "${TGZ_BUILD_ROOT}" ]; then
+			elif [ -n "${TGZ_BUILD_ROOT}" -a $TGZ_USING_ROOT -eq 0 ]; then
 				echo "INSTALLDIR=${PREFIX}" | sed "s|${TGZ_BUILD_ROOT}||g" >>"${RKHINST_ETC_DIR}/${FILE}"
 				echo "DBDIR=${RKHINST_DB_DIR}" | sed "s|${TGZ_BUILD_ROOT}||g" >>"${RKHINST_ETC_DIR}/${FILE}"
 				echo "SCRIPTDIR=${RKHINST_SCRIPT_DIR}" | sed "s|${TGZ_BUILD_ROOT}||g" >>"${RKHINST_ETC_DIR}/${FILE}"
 				echo "TMPDIR=${RKHINST_TMP_DIR}" | sed "s|${TGZ_BUILD_ROOT}||g" >>"${RKHINST_ETC_DIR}/${FILE}"
 				echo "USER_FILEPROP_FILES_DIRS=${RKHINST_ETC_DIR}/${FILE}" | sed "s|${TGZ_BUILD_ROOT}||g" >>"${RKHINST_ETC_DIR}/${FILE}"
-			# Debian builds are handled with a patch during the build process.
-			elif [ -z "${DEB_BUILD_ROOT}" ]; then
-				echo "INSTALLDIR=${PREFIX}" >>"${RKHINST_ETC_DIR}/${FILE}"
+			elif [ -n "${DEB_BUILD_ROOT}" ]; then
+				# Debian builds are handled with a patch during the build process.
+				:
+			else
+				test -z "${PREFIX}" && RKHTMPVAR="/" || RKHTMPVAR="${PREFIX}"
+
+				echo "INSTALLDIR=${RKHTMPVAR}" >>"${RKHINST_ETC_DIR}/${FILE}"
 				echo "DBDIR=${RKHINST_DB_DIR}" >>"${RKHINST_ETC_DIR}/${FILE}"
 				echo "SCRIPTDIR=${RKHINST_SCRIPT_DIR}" >>"${RKHINST_ETC_DIR}/${FILE}"
 				echo "TMPDIR=${RKHINST_TMP_DIR}" >>"${RKHINST_ETC_DIR}/${FILE}"
@@ -819,13 +934,13 @@ doInstall()  {
 			STR=`grep "${PREFIX}" "${FILE}"`
 
 			if [ -n "${STR}" ]; then
-				sed -i "s|${STRIPROOT}||g" "${FILE}"
+				sed -i "s|${STRIPROOT}||g" "${FILE}" >/dev/null 2>&1
 				ERRCODE=$?
 
 				if [ $ERRCODE -eq 0 ]; then
 					echo " Striproot ${FILE}: OK"
 				else
-					echo " Striproot ${FILE}: FAILED: Code $ERRCODE: exiting."
+					echo " Striproot ${FILE}: FAILED: Code $ERRCODE"
 					exit 1
 				fi
 			fi
@@ -857,13 +972,14 @@ doInstall()  {
 
 doRemove()  {
 	RKHINST_DIRS="$RKHINST_ETC_DIR $RKHINST_BIN_DIR $RKHINST_SCRIPT_DIR $RKHINST_DOC_DIR $RKHINST_DB_DIR $RKHINST_TMP_DIR $RKHINST_LANG_DIR"
-	RKHINST_DIRS_POST="$VARDIR $SHAREDIR $PREFIX"
 
 	echo "Starting uninstallation"
 	echo ""
 
 	# Check the PREFIX
-	if [ -d "${PREFIX}" ]; then
+	if [ -z "${PREFIX}" ]; then
+		:
+	elif [ -d "${PREFIX}" ]; then
 		if [ -w "${PREFIX}" ]; then
 			echo "Checking installation directory \"$PREFIX\": it exists and is writable."
 		else
@@ -881,7 +997,7 @@ doRemove()  {
 
 	# Standalone removal involves just deleting the 'files' subdirectory.
 	if [ "$PREFIX" = "." ]; then
-		rm -rf ./files
+		rm -rf ./files >/dev/null 2>&1
 		ERRCODE=$?
 
 		if [ $ERRCODE -eq 0 ]; then
@@ -899,7 +1015,7 @@ doRemove()  {
 	# Man page
 	for FILE in ${RKHINST_MAN_FILES}; do
 		if [ -f "${RKHINST_MAN_DIR}/${FILE}" ]; then
-			rm -f "${RKHINST_MAN_DIR}/${FILE}"
+			rm -f "${RKHINST_MAN_DIR}/${FILE}" >/dev/null 2>&1
 			ERRCODE=$?
 
 			if [ $ERRCODE -eq 0 ]; then
@@ -914,7 +1030,7 @@ doRemove()  {
 	# Application
 	for FILE in ${RKHINST_BIN_FILES}; do
 		if [ -f "${RKHINST_BIN_DIR}/${FILE}" ]; then
-			rm -f "${RKHINST_BIN_DIR}/${FILE}"
+			rm -f "${RKHINST_BIN_DIR}/${FILE}" >/dev/null 2>&1
 			ERRCODE=$?
 
 			if [ $ERRCODE -eq 0 ]; then
@@ -929,7 +1045,7 @@ doRemove()  {
 	# Configuration file
 	for FILE in ${RKHINST_ETC_FILE}; do
 		if [ -f "${RKHINST_ETC_DIR}/${FILE}" ]; then
-			rm -f "${RKHINST_ETC_DIR}/${FILE}"
+			rm -f "${RKHINST_ETC_DIR}/${FILE}" >/dev/null 2>&1
 			ERRCODE=$?
 
 			if [ $ERRCODE -eq 0 ]; then
@@ -956,7 +1072,7 @@ doRemove()  {
 		case "${DIR}" in 
 		*/${APPNAME}) 
 			if [ -d "${DIR}" ]; then
-				rm -rf "${DIR}"
+				rm -rf "${DIR}" >/dev/null 2>&1
 				ERRCODE=$?
 
 				if [ $ERRCODE -eq 0 ]; then
@@ -973,7 +1089,7 @@ doRemove()  {
 
 			for RKHAPPDIR in ${DIR}/${APPNAME}-*; do
 				if [ -d "${RKHAPPDIR}" ]; then
-					rm -rf "${RKHAPPDIR}"
+					rm -rf "${RKHAPPDIR}" >/dev/null 2>&1
 					ERRCODE=$?
 
 					if [ $ERRCODE -eq 0 ]; then
@@ -988,7 +1104,7 @@ doRemove()  {
 			DIR=`dirname "${DIR}"`
 
 			if [ -d "${DIR}" ]; then
-				rm -rf "${DIR}"
+				rm -rf "${DIR}" >/dev/null 2>&1
 				ERRCODE=$?
 
 				if [ $ERRCODE -eq 0 ]; then
@@ -1054,7 +1170,7 @@ while [ $# -ge 1 ]; do
 			if [ -n "$1" ]; then
 				RKHINST_LAYOUT="custom_$1"
 			else
-				echo "No custom layout given, exiting."
+				echo "No custom layout given. Exiting."
 				exit 1
 			fi
 			;;
@@ -1062,7 +1178,7 @@ while [ $# -ge 1 ]; do
 			RKHINST_LAYOUT="$1"
 			;;
 		*)
-			echo "Unknown layout given, exiting: $1"
+			echo "Unknown layout given: $1"
 			exit 1
 			;;
 		esac
@@ -1073,7 +1189,7 @@ while [ $# -ge 1 ]; do
 		if [ -n "$1" ]; then
 			STRIPROOT="$1"
 		else
-			echo "Striproot requested but no directory name given, exiting."
+			echo "Striproot requested but no directory name given. Exiting."
 			exit 1
 		fi
 		;;
@@ -1098,20 +1214,20 @@ done
 
 # We only get here when some installation action was to be taken.
 if [ $RKHINST_ACTION_SEEN -eq 0 ]; then
-	echo "No action given, exiting."
+	echo "No action given. Exiting."
 else
 	case "${RKHINST_ACTION}" in
 	show)
 		showTemplate $RKHINST_LAYOUT
 		;;
 	remove)	# Clean active window
-		clear
 		selectTemplate $RKHINST_LAYOUT
+		clear
 		doRemove
 		;;
 	install) # Clean active window
-		clear
 		selectTemplate $RKHINST_LAYOUT
+		clear
 		doInstall
 		;;
 	esac
